@@ -1,9 +1,9 @@
-/* $Id: tiffgt.c,v 1.6 2006/02/09 16:00:34 dron Exp $ */
+/* $Id: tiffgt.c,v 1.14 2015-06-25 02:28:01 bfriesen Exp $ */
 
 /*
  * Copyright (c) 1988-1997 Sam Leffler
  * Copyright (c) 1991-1997 Silicon Graphics, Inc.
- * Copyright (c) 2003, Andrey Kiselev <dron@remotesensing.org>
+ * Copyright (c) 2003, Andrey Kiselev <dron@ak4719.spb.edu>
  *
  * Permission to use, copy, modify, distribute, and sell this software and 
  * its documentation for any purpose is hereby granted without fee, provided
@@ -31,15 +31,19 @@
 #include <string.h>
 #include <unistd.h>
 
-#if HAVE_APPLE_OPENGL_FRAMEWORK
+#ifdef HAVE_OPENGL_GL_H
 # include <OpenGL/gl.h>
-# include <GLUT/glut.h>
 #else
 # include <GL/gl.h>
+#endif
+#ifdef HAVE_GLUT_GLUT_H
+# include <GLUT/glut.h>
+#else
 # include <GL/glut.h>
 #endif
 
 #include "tiffio.h"
+#include "tiffiop.h"
 
 #ifndef HAVE_GETOPT
 extern int getopt(int, char**, char*);
@@ -73,8 +77,11 @@ static void	raster_reshape(int, int);
 static void	raster_keys(unsigned char, int, int);
 static void	raster_special(int, int, int);
 
+#if !HAVE_DECL_OPTARG
 extern  char* optarg;
 extern  int optind;
+#endif
+
 static TIFF* tif = NULL;
 
 int
@@ -204,7 +211,7 @@ initImage(void)
         if (photo != (uint16) -1)
                 TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, photo);
         if (!TIFFRGBAImageBegin(&img, tif, stoponerr, title)) {
-                TIFFError(filelist[fileindex], title);
+                TIFFError(filelist[fileindex], "%s", title);
                 TIFFClose(tif);
                 tif = NULL;
                 return -1;
@@ -224,21 +231,24 @@ initImage(void)
                 w = xmax;
         }
 
-        if (w != width || h != height) {
-            if (raster != NULL)
-                _TIFFfree(raster), raster = NULL;
-            raster = (uint32*) _TIFFmalloc(img.width * img.height * sizeof (uint32));
-            if (raster == NULL) {
-                width = height = 0;
-                TIFFError(filelist[fileindex], "No space for raster buffer");
-                cleanup_and_exit();
-            }
-            width = w;
-            height = h;
-        }
-        TIFFRGBAImageGet(&img, raster, img.width, img.height);
+	if (w != width || h != height) {
+		uint32 rastersize =
+			_TIFFMultiply32(tif, img.width, img.height, "allocating raster buffer");
+		if (raster != NULL)
+			_TIFFfree(raster), raster = NULL;
+		raster = (uint32*) _TIFFCheckMalloc(tif, rastersize, sizeof (uint32),
+						    "allocating raster buffer");
+		if (raster == NULL) {
+			width = height = 0;
+			TIFFError(filelist[fileindex], "No space for raster buffer");
+			cleanup_and_exit();
+		}
+		width = w;
+		height = h;
+	}
+	TIFFRGBAImageGet(&img, raster, img.width, img.height);
 #if HOST_BIGENDIAN
-        TIFFSwabArrayOfLong(raster,img.width*img.height);
+	TIFFSwabArrayOfLong(raster,img.width*img.height);
 #endif
 	return 0;
 }
@@ -283,6 +293,7 @@ static void
 raster_draw(void)
 {
   glDrawPixels(img.width, img.height, GL_RGBA, GL_UNSIGNED_BYTE, (const GLvoid *) raster);
+  glFlush();
 }
 
 static void
@@ -302,6 +313,8 @@ raster_reshape(int win_w, int win_h)
 static void
 raster_keys(unsigned char key, int x, int y)
 {
+        (void) x;
+        (void) y;
         switch (key) {
                 case 'b':                       /* photometric MinIsBlack */
                     photo = PHOTOMETRIC_MINISBLACK;
@@ -347,6 +360,8 @@ raster_keys(unsigned char key, int x, int y)
 static void
 raster_special(int key, int x, int y)
 {
+        (void) x;
+        (void) y;
         switch (key) {
                 case GLUT_KEY_PAGE_UP:          /* previous logical image */
                     if (TIFFCurrentDirectory(tif) > 0) {
@@ -453,3 +468,10 @@ photoArg(const char* arg)
 }
 
 /* vim: set ts=8 sts=8 sw=8 noet: */
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 8
+ * fill-column: 78
+ * End:
+ */
